#!/usr/bin/perl -w

# Copyright (c) 2002 ISOMEDIA, Inc.
# Written by Steve Milton
# Released under the GNU Public License
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Usage:   check_raid <raid-name>
# Example: check_raid md0
#          WARNING md0 status=[UUU_U], recovery=46.4%, finish=123.0min

use strict;
use Getopt::Long qw(:config no_ignore_case);

my %ERRORS=('DEPENDENT'=>4,'UNKNOWN'=>3,'OK'=>0,'WARNING'=>1,'CRITICAL'=>2);

my $mdstat = "/proc/mdstat";
my $verbose = 0;

GetOptions('mdstat=s' => \$mdstat,
	   'v|verbose+' => \$verbose,
	   'h|help' => sub { usage();  exit $ERRORS{'UNKNOWN'}; },
	   'V|version' => sub { version();  exit $ERRORS{'UNKNOWN'}; });

# die with an error if we're not on Linux
if ($^O ne 'linux') {
    print "This plugin only applicable on Linux.\n";
    exit $ERRORS{'UNKNOWN'};
}

my $mddev = shift;

open (MDSTAT, "<$mdstat") or die "Failed to open $mdstat";
my $found = 0;
my $status = "";
my $recovery = "";
my $finish = "";
my $speed = "";
my $active = "";
while(<MDSTAT>) {
    chomp;
    if ($found) {
        if (/(\[[_U]+\])/) {
            $status = $1;
        } elsif (/recovery *= *([^ ]+)/) {
            $recovery = $1;
            ($finish) = /finish= *([^ ]+)/;
            ($speed) = /speed= *([^ ]+)/;
        } elsif (/^[^ \t]/) {
            last;
        }
    } else {
        if (/^$mddev\s*:/) {
            $found = 1;
            if (/active/) {
                $active = 1;
            }
        }
    }
}

my $msg = "FAILURE";
my $code = "UNKNOWN";
if ($status =~ /_/) {
    if ($recovery) {
        $msg = "$mddev status=$status, recovery=$recovery, finish=$finish, speed=$speed",
        $code = "WARNING";
    } else {
        $msg = "$mddev status=$status";
        $code = "CRITICAL";
    }
} elsif ($status =~ /U+/) {
    $msg = "$mddev status=$status";
    $code = "OK";
} else {
    if ($active) {
        $msg = "$mddev active with no status information";
        $code = "OK";
    } else {
        $msg = "$mddev does not exist";
        $code = "CRITICAL";
    }
}

print "RAID $code: $msg\n";
exit ($ERRORS{$code});

sub usage {
    print qq{usage:
    check_linux_raid.pl [--mdstat=</proc/mdstat>] <md-device>
e.g.
    check_linux_raid.pl md0
};
}

sub version {
    print "check_linux_raid.pl 1.1\n";
}
